/*
 * Quasseldroid - Quassel client for Android
 *
 * Copyright (c) 2021 Janne Mareike Koschinski
 * Copyright (c) 2021 The Quassel Project
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package de.justjanne.libquassel.protocol.types

import de.justjanne.libquassel.annotations.Generated
import java.io.Serializable

/**
 * Native representation of a SignedId
 */
typealias SignedIdType = Int
/**
 * Native representation of a SignedId64
 */
typealias SignedId64Type = Long

/**
 * Base type for a signed id
 */
interface SignedId<T> : Serializable, Comparable<SignedId<T>>
  where T : Number, T : Comparable<T> {
  /**
   * Underlying representation
   */
  val id: T

  override fun compareTo(other: SignedId<T>): Int {
    return id.compareTo(other.id)
  }
}

/**
 * Validity check for a signed id
 */
@Suppress("NOTHING_TO_INLINE")
@JvmName("isValidId")
@Generated
inline fun SignedId<SignedIdType>.isValid() = id > 0

/**
 * Validity check for a signed id
 */
@Suppress("NOTHING_TO_INLINE")
@JvmName("isValidId64")
@Generated
inline fun SignedId<SignedId64Type>.isValid() = id > 0
